#coding: utf-8
import os
import subprocess
import threading

import wx


class CodecArea():
    def __init__(self, frame):
        self.frame = frame
        self.mainPanel = None
        self.codeCFileText = None                         # 转码文件地址
        self.saveFileText = None                          # 要保存的文件地址
        self.selectVcodecCombox = None                    # 选择视频编码器
        self.selectVresolutionCombox = None               # 选择分辨率
        self.selectFrameRateCombox = None                 # 选择帧率
        self.selectAcodecCombox = None                    # 选择音频编码器


    #################################################
    #       创建一个顶级pannel
    #################################################
    def create(self):
        self.mainPanel = wx.Panel(self.frame)
        boxSizer = wx.BoxSizer(wx.VERTICAL)
        topPanel = wx.Panel(self.mainPanel)
        # topPanel.SetBackgroundColour("YELLOW")
        fiPath = os.getcwd() + "/flv/aaa3.flv"
        wx.StaticText(topPanel, label='转码文件：', pos=(10, 10))
        self.codeCFileText = wx.TextCtrl(topPanel, pos=(70, 5), size=wx.Size(300, -1), value=fiPath)
        selectCodecPathButton = wx.Button(topPanel, label="选择视频文件", pos=(375, 5))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt, textCtr=self.codeCFileText: self.selectVideoFile(evt, textCtr),
                        selectCodecPathButton)

        wx.StaticText(topPanel, label='保存文件：', pos=(480, 10))
        self.saveFileText = wx.TextCtrl(topPanel, pos=(540, 5), size=wx.Size(300, -1), value="")
        selectSavePathButton = wx.Button(topPanel, label="选择目录", pos=(845, 5))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt, textCtr=self.saveFileText: self.saveVideoFile(evt, textCtr),
                        selectSavePathButton)
        startCodecButton = wx.Button(topPanel, label="开始转码", pos=(930, 5))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.startCodec(evt),
                        startCodecButton)
        wx.StaticText(topPanel, label='视频编码器：', pos=(10, 40))
        self.selectVcodecCombox = wx.ComboBox(topPanel, pos=(90, 35), choices=['libx264', 'copy'], value="libx264",
                                           size=wx.Size(100, -1))
        wx.StaticText(topPanel, label='视频分辨率：', pos=(200, 40))
        self.selectVresolutionCombox = wx.ComboBox(topPanel, pos=(280, 35), choices=['1080x720', '720x480','640x426','480x320','320x214'], value="640x426",
                                           size=wx.Size(80, -1))
        wx.StaticText(topPanel, label='视频帧率：', pos=(370, 40))
        self.selectFrameRateCombox = wx.ComboBox(topPanel, pos=(440, 35), choices=['25', '30'], value="25",
                                           size=wx.Size(70, -1))
        wx.StaticText(topPanel, label='音频编码器：', pos=(10, 70))
        self.selectAcodecCombox = wx.ComboBox(topPanel, pos=(90, 65), choices=['aac', 'copy'], value="aac",
                                           size=wx.Size(100, -1))


        bottomPanel = wx.Panel(self.mainPanel)
        boxSizer.Add(topPanel,1,flag=wx.EXPAND | wx.ALL)
        boxSizer.Add(bottomPanel,2, flag=wx.EXPAND | wx.TOP , border=5)
        self.mainPanel.SetSizer(boxSizer)
        return self.mainPanel

    #################################################
    #       打开文件选择对话框
    #################################################
    def selectVideoFile(self,evt,textCtr):
        wildcard = "Video file (*.flv;*.mp4;*.mkv;*.avi)|*.flv;*.mp4;*.mkv;*.avi"
        dlg = wx.FileDialog(
            self.frame, message="选择视频文件",
            defaultDir=os.getcwd(),
            defaultFile="",
            wildcard=wildcard,
            style=wx.FD_OPEN |
                  wx.FD_CHANGE_DIR | wx.FD_FILE_MUST_EXIST |
                  wx.FD_PREVIEW
        )
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            textCtr.SetValue(path)
        dlg.Destroy()

    #################################################
    #       打开文件选择对话框
    #################################################
    def saveVideoFile(self,evt,textCtr):
        wildcard = "Video file (*.flv)|*.flv"
        dlg = wx.FileDialog(
            self.frame, message="保存文件",
            defaultDir=os.getcwd(),
            defaultFile="",
            wildcard=wildcard,
            style=wx.FD_SAVE |
                  wx.FD_CHANGE_DIR | wx.FD_PREVIEW
        )
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            textCtr.SetValue(path)
        dlg.Destroy()


    #################################################
    #       开始转码
    #################################################
    def startCodec(self,evt):
        srcVideo = self.codeCFileText.GetValue()
        dstFile = self.saveFileText.GetValue()
        if dstFile == "":
            dstFile = srcVideo[:len(srcVideo) - 3] + "flv"
        vcodec = self.selectVcodecCombox.GetValue()
        vresolution = self.selectVresolutionCombox.GetValue()
        frameRate = self.selectFrameRateCombox.GetValue()
        acodec = self.selectAcodecCombox.GetValue()
        threadObj = threading.Thread(target=self.codec,args=(srcVideo,vcodec,vresolution,frameRate,acodec,dstFile,))
        threadObj.start()
    def codec(self,srcVideo,vcodec,vresolution,frameRate,acodec,dstFile):
        m_command = ["ffmpeg.exe", "-i",srcVideo,"-vcodec",vcodec,"-s",vresolution,"-r",frameRate,"-acodec",acodec,dstFile]
        subprocess.run(m_command)


